'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.removeCache = removeCache;
exports.default = createBullQueue;

var _bull = require('bull');

var _bull2 = _interopRequireDefault(_bull);

var _priorityQueue = require('bull/lib/priority-queue');

var _priorityQueue2 = _interopRequireDefault(_priorityQueue);

var _redis = require('redis');

var _redis2 = _interopRequireDefault(_redis);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_redis2.default.RedisClient.prototype);
_bluebird2.default.promisifyAll(_redis2.default.Multi.prototype);

var REDIS_PORT = process.env.REDIS_PORT || 6379;
var REDIS_HOST = process.env.REDIS_HOST || '127.0.0.1';

var redisClient = _redis2.default.createClient({
  host: REDIS_HOST,
  port: REDIS_PORT
});

var KEY_PREFIX = 'scraper.js-queue-bull-url-cache';

function getKey(queueItem) {
  return KEY_PREFIX + ':(' + queueItem.url + ')';
}

function removeCache(queueItem) {
  return redisClient.delAsync(getKey(queueItem));
}

function createBullQueue(name) {
  var createQueue = process.env.PRIORITY_QUEUE ? _priorityQueue2.default : _bull2.default;
  var queue = createQueue(name, REDIS_PORT, REDIS_HOST);

  return {
    process: function process(options, fn) {
      queue.process(options, function (job) {
        return fn(job.data, job);
      });
    },
    empty: function empty() {
      return queue.empty();
    },
    add: function add(queueItem) {
      var _this = this;

      return (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var _queueItem$expiry, expiry, _queueItem$priority, priority, _queueItem$attempts, attempts, _queueItem$backoff, backoff, key, saved;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _queueItem$expiry = queueItem.expiry, expiry = _queueItem$expiry === undefined ? 86400000 : _queueItem$expiry, _queueItem$priority = queueItem.priority, priority = _queueItem$priority === undefined ? 'normal' : _queueItem$priority, _queueItem$attempts = queueItem.attempts, attempts = _queueItem$attempts === undefined ? 2 : _queueItem$attempts, _queueItem$backoff = queueItem.backoff, backoff = _queueItem$backoff === undefined ? 3600000 : _queueItem$backoff;

                // @todo was doing: removing options paramater and combining it with the queueItem

                key = KEY_PREFIX + ':(' + queueItem.url + ')';
                _context.next = 4;
                return redisClient.setnxAsync(key, 'true');

              case 4:
                saved = _context.sent;

                if (!(saved === 1)) {
                  _context.next = 11;
                  break;
                }

                _context.next = 8;
                return queue.add(queueItem, {
                  priority: priority,
                  attempts: attempts,
                  backoff: backoff
                });

              case 8:
                _context.next = 10;
                return redisClient.expireAsync(key, Math.ceil(expiry / 1000));

              case 10:
                return _context.abrupt('return', true);

              case 11:
                return _context.abrupt('return', false);

              case 12:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }))();
    },
    count: function count() {
      return queue.count();
    },
    close: function close() {
      return queue.close();
    }
  };
}